part of firebase_model_notifier.dynamic_links.mobile;

final firebaseDynamicLinksUrlProvider =
    ChangeNotifierProvider.family<FirebaseDynamicLinksUrlModel, String>(
  (_, path) => FirebaseDynamicLinksUrlModel(path),
);

class FirebaseDynamicLinksUrlModel extends ValueModel<String?> {
  FirebaseDynamicLinksUrlModel(this.path) : super(null);

  /// Access path.
  final String path;

  /// Returns itself after the creating finishes.
  Future<void>? get creating => _createCompleter?.future;
  Completer<void>? _createCompleter;

  static const String restAPIURL =
      "https://firebasedynamiclinks.googleapis.com/v1/shortLinks?key=%s";

  /// Create a URL for Dynamic Link.
  ///
  /// Create a URL to access [path].
  ///
  /// [path]: Path to create.
  /// [options]: Firestore dynamiclink options.
  /// [socialTitle]: Social media title.
  /// [socialImageURL]: Social image url.
  /// [socialDescription]: Social media description.
  Future<String> create({
    required FirebaseDynamicLinksOptions options,
    String? socialDescription,
    String? socialImageURL,
    String? socialTitle,
  }) async {
    assert(
      options.webAPIKey != null,
      "In the case of Web, [webAPIKey] must be entered.",
    );
    assert(
      path.isNotEmpty && options.uriPrefix.isNotEmpty,
      "Path must not be empty.",
    );
    assert(
      options.appStoreId.isNotEmpty && options.packageName.isNotEmpty,
      "Bundle Id / Package must not be empty.",
    );
    await _constructURI(
      uriPrefix: options.uriPrefix,
      appStoreId: options.appStoreId,
      webAPIKey: options.webAPIKey,
      packageName: options.packageName,
      bundleId: options.bundleId,
      minimumVersion: options.minimumVersion,
      socialDescription: socialDescription ?? options.defaultSocialDescription,
      socialImageURL: socialImageURL ?? options.defaultSocialImageURL,
      socialTitle: socialTitle ?? options.defaultSocialTitle,
    );
    return value ?? "";
  }

  Future<void> _constructURI({
    required String uriPrefix,
    required String appStoreId,
    String? webAPIKey,
    required String packageName,
    String? bundleId,
    int minimumVersion = 1,
    String? socialDescription,
    String? socialImageURL,
    String? socialTitle,
  }) async {
    if (value.isNotEmpty) {
      return;
    }
    if (_createCompleter != null) {
      return creating;
    }
    try {
      final data = DynamicLinkParameters(
        uriPrefix: uriPrefix,
        link: Uri.parse(path),
        androidParameters: AndroidParameters(
          packageName: packageName,
          minimumVersion: minimumVersion,
        ),
        iosParameters: IosParameters(
          bundleId: bundleId ?? packageName,
          minimumVersion: minimumVersion.toString(),
          appStoreId: appStoreId,
        ),
        socialMetaTagParameters: SocialMetaTagParameters(
          title: socialTitle?.localize(),
          description: socialDescription?.localize(),
          imageUrl: socialImageURL != null ? Uri.parse(socialImageURL) : null,
        ),
      );
      value = ((await data.buildShortLink()).shortUrl).toString();
      _createCompleter?.complete();
      _createCompleter = null;
    } catch (e) {
      _createCompleter?.completeError(e);
      _createCompleter = null;
      rethrow;
    } finally {
      _createCompleter?.complete();
      _createCompleter = null;
    }
  }
}
