part of firebase_model_notifier;

/// GeoData for Firestore.
@immutable
class FirebaseGeoData extends GeoData {
  /// GeoData for Firestore.
  FirebaseGeoData({
    double latitude = 0,
    double longitude = 0,
    String? name,
  })  : _geoFirePoint = GeoFirePoint(latitude, longitude),
        _name = name ?? "";

  /// GeoData for Firestore.
  FirebaseGeoData.fromGeoPoint(GeoPoint geoPoint, {String? name})
      : this(
          latitude: geoPoint.latitude,
          longitude: geoPoint.longitude,
          name: name,
        );

  /// GeoData for Firestore.
  FirebaseGeoData.fromGeoData(GeoData geoData, {String? name})
      : this(
          latitude: geoData.latitude,
          longitude: geoData.longitude,
          name: name ?? geoData.name,
        );

  final GeoFirePoint _geoFirePoint;

  /// Location name.
  @override
  String get name {
    if (_name.isNotEmpty) {
      return _name;
    }
    return "$latitude,$longitude";
  }

  final String _name;

  /// Latitude.
  @override
  double get latitude => _geoFirePoint.latitude;

  /// Longitude.
  @override
  double get longitude => _geoFirePoint.longitude;

  /// Geographical distance between two Co-ordinates.
  static double distanceBetween({
    required Coordinates to,
    required Coordinates from,
  }) {
    return GeoFirePoint.distanceBetween(to: to, from: from);
  }

  /// Neighboring geo-hashes of [hash].
  static List<String> neighborsOf({required String hash}) {
    return GeoFirePoint.neighborsOf(hash: hash);
  }

  /// Hash of [GeoData].
  @override
  String get hash {
    return _geoFirePoint.hash;
  }

  /// All neighbors of [GeoData].
  @override
  List<String> get neighbors {
    return _geoFirePoint.neighbors;
  }

  /// [GeoPoint] of [GeoData].
  GeoPoint get geoPoint {
    return _geoFirePoint.geoPoint;
  }

  /// [Coorinates] of [GeoData].
  Coordinates get coords {
    return _geoFirePoint.coords;
  }

  /// Set up a Firestore query.
  ///
  /// It is possible to query the collection by setting this in FirestoreCollection.
  @override
  List<String> regionHash([double radius = 1.0]) {
    if (radius <= 0) {
      return const [];
    }
    final precision = _GeoUtility.setPrecision(radius);
    final centerHash = hash.substring(0, precision);
    return GeoFirePoint.neighborsOf(hash: centerHash)..add(centerHash);
  }

  /// Calculate the distance between two points.
  ///
  /// [data]: GeoData.
  @override
  double distance(GeoData data) {
    return distanceBetween(
      from: coords,
      to: Coordinates(data.latitude, data.longitude),
    );
  }

  /// Raw data.
  DynamicMap get data {
    return {'geopoint': geoPoint, 'geohash': hash};
  }
}
